@@ -1,23 +1,44 @@
 #ifndef socket_h
 #define socket_h
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <arpa/inet.h>
 #include <netinet/in.h>
+#include <sys/socket.h>
+#include <string>
 #include <vector>
 #include <list>
 
-#include "pthread_rdwr.h"
-
 #include "licq_buffer.h"
 #include "licq_constants.h"
+#include "licq_mutex.h"
+#include "licq_types.h"
 
 class ProxyServer;
 
-char *inet_ntoa_r(struct in_addr in, char *buf);
+// IPv6 is implemented in socket classes but Licq support must be considered experimental for now
+// Uncomment the following line to disable IPv6 socket usage
+//#define LICQ_DISABLE_IPV6
+
+
+// Define for marking functions as deprecated
+#ifndef LICQ_DEPRECATED
+# if defined(__GNUC__) && !defined(__INTEL_COMPILER) && (__GNUC__ - 0 > 3 || (__GNUC__ - 0 == 3 && __GNUC_MINOR__ - 0 >= 2))
+#  define LICQ_DEPRECATED __attribute__ ((__deprecated__))
+# elif defined(_MSC_VER) && (_MSC_VER >= 1300)
+#  define LICQ_DEPRECATED __declspec(deprecated)
+# else
+#  define LICQ_DEPRECATED
+# endif
+#endif
+
+
+/**
+ * Convert an IPv4 address to readable text
+ *
+ * @param in IP address
+ * @param buf Buffer for write text to, assumed to be able to hold at least 32 bytes
+ * @return buf
+ */
 char *ip_ntoa(unsigned long in, char *buf);
 
 typedef enum SocketError_et_
@@ -35,39 +56,106 @@ typedef enum SocketError_et_
 class INetSocket
 {
 public:
-  INetSocket(const char *_szOwnerId, unsigned long _nOwnerPPID);
+  INetSocket(const UserId& userId);
   virtual ~INetSocket();
 
   bool Connected()          { return(m_nDescriptor > 0);  }
   int Descriptor()          { return(m_nDescriptor);      }
-  bool DestinationSet()     { return (RemoteIp() != 0); }
-  char *OwnerId()           { return m_szOwnerId; }
-  unsigned long OwnerPPID() { return m_nOwnerPPID; }
-  void SetOwner(const char *s, unsigned long n);
+  bool DestinationSet()     { return myRemoteAddr.sa_family != AF_UNSPEC; }
+  const UserId& userId() const { return myUserId; }
+  void setUserId(const UserId& userId) { myUserId = userId; }
   unsigned long Version()     { return (m_nVersion); }
   void SetVersion(unsigned long _nVersion)  { m_nVersion = _nVersion; }
 
   int Error();
   char *ErrorStr(char *, int);
 
-  unsigned long  LocalIp()     { return (m_sLocalAddr.sin_addr.s_addr);  }
-  char *LocalIpStr(char *buf);
-  unsigned long  RemoteIp()    { return (m_sRemoteAddr.sin_addr.s_addr); };
-  char *RemoteIpStr(char *buf);
-  unsigned short LocalPort()   { return (ntohs(m_sLocalAddr.sin_port));  };
-  unsigned short RemotePort()  { return (ntohs(m_sRemoteAddr.sin_port)); };
+  /**
+   * Get IP address of local endpoint as a readable string
+   *
+   * @return Local IP address
+   */
+  std::string getLocalIpString() const  { return addrToString(&myLocalAddr); }
 
-  bool SetRemoteAddr(unsigned long _nRemoteIp, unsigned short _nRemotePort);
-  bool SetRemoteAddr(const char *_szRemoteName, unsigned short _nRemotePort);
+  /**
+   * Get IP address of remote endpoint as a readable string
+   *
+   * @return Remote IP address
+   */
+  std::string getRemoteIpString() const { return addrToString(&myRemoteAddr); }
 
-  void SetProxy(ProxyServer *_xProxy) { m_xProxy = _xProxy; };
+  /**
+   * Get IP address of local endpoint as an unsigned int
+   * Note: This function is not usable with IPv6 sockets
+   *
+   * @return Local IP address in network endian form
+   */
+  uint32_t getLocalIpInt() const        { return addrToInt(&myLocalAddr); }
+
+  /**
+   * Get IP address of remote endpoint as an unsigned int
+   * Note: This function is not usable with IPv6 sockets
+   *
+   * @return Remote IP address in network endian form
+   */
+  uint32_t getRemoteIpInt() const       { return addrToInt(&myRemoteAddr); }
+
+  /**
+   * Get local port number
+   *
+   * @return Local port number in host endian form
+   */
+  uint16_t getLocalPort() const         { return getAddrPort(&myLocalAddr); }
+
+  /**
+   * Get remote port number
+   *
+   * @return Remote port number in host endian form
+   */
+  uint16_t getRemotePort() const        { return getAddrPort(&myRemoteAddr); };
+
+
+  LICQ_DEPRECATED // Use getLocalIpInt() instead
+  unsigned long  LocalIp() const        { return getLocalIpInt(); }
+  LICQ_DEPRECATED // Use getLocalIpString() instead
+  char *LocalIpStr(char *buf) const;
+  LICQ_DEPRECATED // Use getRemoteIpInt() instead
+  unsigned long  RemoteIp() const       { return getRemoteIpInt(); }
+  LICQ_DEPRECATED // Use getRemoteIpString() instead
+  char *RemoteIpStr(char *buf) const;
+  LICQ_DEPRECATED // Use getLocalPort() instead
+  unsigned short LocalPort()            { return getLocalPort(); }
+  LICQ_DEPRECATED // Use getRemotePort() instead
+  unsigned short RemotePort()           { return getRemotePort(); }
 
   void ResetSocket();
   void ClearRecvBuffer()  { m_xRecvBuffer.Clear(); };
   bool RecvBufferFull()   { return m_xRecvBuffer.Full(); };
   CBuffer &RecvBuffer()   { return m_xRecvBuffer; };
 
-  bool OpenConnection();
+  /**
+   * Connect to a remote host
+   *
+   * @param remoteAddr Address of remote host
+   * @param remotePort Port to connect to
+   * @param proxy Proxy connection to use or NULL for direct connect
+   * @return True if connection was opened successfully
+   */
+  bool connectTo(const std::string& remoteAddr, uint16_t remotePort,
+      ProxyServer* proxy = NULL);
+
+  /**
+   * Connect to a remote IP as unsigned int
+   * Note: This function is not usable with IPv6
+   *
+   * @param remoteAddr Address of remote host
+   * @param remotePort Port to connect to
+   * @param proxy Proxy connection to use or NULL for direct connect
+   * @return True if connection was opened successfully
+   */
+  bool connectTo(uint32_t remoteAddr, uint16_t remotePort,
+      ProxyServer* proxy = NULL);
+
   void CloseConnection();
   bool StartServer(unsigned int _nPort);
   bool SendRaw(CBuffer *b);
@@ -80,7 +168,30 @@ public:
   void Unlock();
   pthread_mutex_t mutex;
 
-  static unsigned long GetIpByName(const char *_szHostName);
+  /**
+   * Convert an address to readable string form
+   *
+   * @param addr A sockaddr_in or sockaddr_in6 with the address to convert
+   * @return The address in text form
+   */
+  static std::string addrToString(const struct sockaddr* addr);
+
+  /**
+   * Get the IPv4 address as an unsigned int from an address
+   * Note: This function is not usable with IPv6 addresses
+   *
+   * @param addr A sockaddr_in with the address to extract
+   * @return The address from the struct in network endian form
+   */
+  static uint32_t addrToInt(const struct sockaddr* addr);
+
+  /**
+   * Get the port from an address
+   *
+   * @param addr A sockaddr_in or sockaddr_in6 with the port to extract
+   * @return The port from the struct
+   */
+  static uint16_t getAddrPort(const struct sockaddr* addr);
 
   void SetChannel(unsigned char nChannel) { m_nChannel = nChannel; }
   unsigned char Channel()                 { return m_nChannel; }
@@ -90,17 +201,27 @@ protected:
   bool SetLocalAddress(bool bIp = true);
   void DumpPacket(CBuffer *b, direction d);
 
+  // sockaddr is too small to hold a sockaddr_in6 so use union to allocate the extra space
+  union
+  {
+    struct sockaddr myLocalAddr;
+    struct sockaddr_storage myLocalAddrStorage;
+  };
+  union
+  {
+    struct sockaddr myRemoteAddr;
+    struct sockaddr_storage myRemoteAddrStorage;
+  };
+
   int m_nDescriptor;
-  struct sockaddr_in m_sRemoteAddr, m_sLocalAddr;
-  char *m_szRemoteName;
+  std::string myRemoteName;
   CBuffer m_xRecvBuffer;
   char m_szID[4];
   int m_nSockType;
   unsigned short m_nVersion;
   SocketError_et m_nErrorType;
   ProxyServer *m_xProxy;
-  char *m_szOwnerId;
-  unsigned long m_nOwnerPPID;
+  UserId myUserId;
   unsigned char m_nChannel;
 };
 
@@ -109,7 +230,7 @@ protected:
 class TCPSocket : public INetSocket
 {
 public:
-  TCPSocket(const char *s, unsigned long n);
+  TCPSocket(const UserId& userId);
   TCPSocket();
   virtual ~TCPSocket();
 
@@ -145,7 +266,7 @@ protected:
 class SrvSocket : public INetSocket
 {
 public:
-  SrvSocket(const char *s, unsigned long n);
+  SrvSocket(const UserId& userId);
   virtual ~SrvSocket();
 
   // Abstract base class overloads
@@ -157,7 +278,6 @@ public:
   // Functions specific to Server TCP communication
   bool SendPacket(CBuffer *b);
   bool RecvPacket();
-  bool ConnectTo(const char* server, unsigned short port, ProxyServer *xProxy);
 };
 
 
@@ -165,7 +285,7 @@ public:
 class UDPSocket : public INetSocket
 {
 public:
-  UDPSocket(const char* ownerId, unsigned long ownerPpid);
+  UDPSocket(const UserId& userId);
   virtual ~UDPSocket();
 
   // Abstract base class overloads
@@ -191,11 +311,7 @@ protected:
   unsigned short HashValue(int _nSd);
   std::vector < std::list<INetSocket *> > m_vlTable;
 
-  void Lock(unsigned short _nLockType);
-  void Unlock();
-
-  pthread_rdwr_t mutex_rw;
-  unsigned short m_nLockType;
+  ReadWriteMutex myMutex;
 };
 
 
