@@ -2,7 +2,9 @@
 #include "config.h"
 #endif
 
-#include <stdio.h>
+#include <cstdio>
+#include <cstring>
+#include <cstdlib>
 #include <unistd.h>
 #include "licq_plugin.h"
 #include "autoreply.h"
@@ -12,7 +14,7 @@ CLicqAutoReply *licqAutoReply;
 const char *LP_Usage()
 {
   static const char usage[] =
-    "Usage:  Licq [options] -p autoreply -- [ -h ] [ -e ] [ -l <staus> ] [ -d ]\n"
+    "Usage:  Licq [options] -p autoreply -- [ -h ] [ -e ] [ -l <status> ] [ -d ]\n"
     "         -h          : help\n"
     "         -e          : start enabled\n"
     "         -l <status> : log on at startup\n"
@@ -41,7 +43,7 @@ const char *LP_Description()
 
 const char *LP_Version()
 {
-  static const char version[] = "1.0.2";
+  static const char version[] = VERSION;
   return version;
 }
 
