@@ -475,7 +475,7 @@ void CLicqConsole::ProcessPipe()
   {
   case 'S':  // A signal is pending
     {
-      CICQSignal *s = licqDaemon->PopPluginSignal();
+      LicqSignal* s = licqDaemon->popPluginSignal();
       ProcessSignal(s);
       break;
     }
@@ -507,7 +507,7 @@ void CLicqConsole::ProcessPipe()
 /*---------------------------------------------------------------------------
  * CLicqConsole::ProcessSignal
  *-------------------------------------------------------------------------*/
-void CLicqConsole::ProcessSignal(CICQSignal *s)
+void CLicqConsole::ProcessSignal(LicqSignal* s)
 {
   switch (s->Signal())
   {
@@ -516,8 +516,8 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
     {
       for (unsigned short i = 0; i < MAX_CON; i++)
       {
-        if (s->Id() == winCon[i]->myLastId)
-          winCon[i]->myLastId.clear();
+        if (s->userId() == winCon[i]->sLastContact)
+            winCon[i]->sLastContact = USERID_NONE;
       }
     }
     PrintStatus();
@@ -526,10 +526,10 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
     break;
   case SIGNAL_UPDATExUSER:
     {
-      if ((gUserManager.FindOwner(s->Id(), s->PPID()) != NULL && s->SubSignal() == USER_STATUS)
+      if ((gUserManager.isOwner(s->userId()) && s->SubSignal() == USER_STATUS)
           || s->SubSignal() == USER_EVENTS)
         PrintStatus();
-      ICQUser *u = gUserManager.FetchUser(s->Id(), s->PPID(), LOCK_R);
+      LicqUser* u = gUserManager.fetchUser(s->userId(), LOCK_R);
       if (u != NULL)
       {
         bool isInGroup = u->GetInGroup(m_nGroupType, m_nCurrentGroup);
@@ -549,13 +549,13 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
     PrintStatus();
     break;
   case SIGNAL_ADDxSERVERxLIST:
-    licqDaemon->icqRenameUser(s->Id());
-    break;
+      licqDaemon->updateUserAlias(s->userId());
+      break;
   case SIGNAL_NEWxPROTO_PLUGIN:
     //ignore for now
     break;
   case SIGNAL_EVENTxID:
-    AddEventTag(s->Id(), s->PPID(), s->Argument());
+      AddEventTag(s->userId(), s->Argument());
     break;
   default:
     gLog.Warn("%sInternal error: CLicqConsole::ProcessSignal(): Unknown signal command received from daemon: %ld.\n",
@@ -568,9 +568,9 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
 /*---------------------------------------------------------------------------
  * CLicqConsole::AddEventTag
  *-------------------------------------------------------------------------*/
-void CLicqConsole::AddEventTag(const char* _szId, unsigned long _nPPID, unsigned long _nEventTag)
+void CLicqConsole::AddEventTag(const UserId& userId, unsigned long _nEventTag)
 {
-  if (!_szId || !_nPPID || !_nEventTag)
+  if (!USERID_ISVALID(userId) || !_nEventTag)
     return;
 
   CData *data;
@@ -578,7 +578,7 @@ void CLicqConsole::AddEventTag(const char* _szId, unsigned long _nPPID, unsigned
   for (i = 1; i <= MAX_CON; i++)
   {
     data = (CData *)winCon[i]->data;
-    if (data && strcmp(data->szId, _szId) == 0 && data->nPPID == _nPPID)
+    if (data && data->userId == userId)
     {
       winCon[i]->event = _nEventTag;
       break;
@@ -810,18 +810,18 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
     {
     case ICQ_CMDxTCP_START:
       {
-        ICQUser *u = NULL;
+        LicqUser* u = NULL;
         const CUserEvent* ue = e->UserEvent();
         if (e->SubResult() == ICQ_TCPxACK_RETURN)
         {
-          u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
+          u = gUserManager.fetchUser(e->userId());
           win->wprintf("%s is in %s mode:\n%s\n[Send \"urgent\" ('.u') to ignore]\n",
                        u->GetAlias(), u->StatusStr(), u->AutoResponse());
           gUserManager.DropUser(u);
         }
         else if (e->SubResult() == ICQ_TCPxACK_REFUSE)
         {
-          u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
+          u = gUserManager.fetchUser(e->userId());
           win->wprintf("%s refused %s.\n",
                        u->GetAlias(), ue->Description());
           gUserManager.DropUser(u);
@@ -838,7 +838,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
 
           if(!ea->Accepted())
           {
-            u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
+            u = gUserManager.fetchUser(e->userId());
             win->wprintf("%s refused file: %s\n",
                          u->GetAlias(), ea->Response());
             gUserManager.DropUser(u);
@@ -850,7 +850,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
             // Invoke a file transfer manager here
             const CEventFile* f = dynamic_cast<const CEventFile *>(ue);
             CFileTransferManager *ftman = new CFileTransferManager(licqDaemon,
-                                          e->Uin());
+                LicqUser::getUserAccountId(e->userId()).c_str());
             m_lFileStat.push_back(ftman);
 
             // Now watch the file pipe
@@ -905,7 +905,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
             } // if file or chat*/
         else
         {
-          u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
+          u = gUserManager.fetchUser(e->userId());
           if (u != NULL && u->Away() && u->ShowAwayMsg())
           {
             win->wprintf("%s\n", u->AutoResponse());
@@ -962,9 +962,9 @@ void CLicqConsole::ProcessDoneSearch(ICQEvent *e)
     return;
   }
 
-  if (e->SearchAck() != NULL && e->SearchAck()->Uin() != 0)
+  if (e->SearchAck() != NULL && USERID_ISVALID(e->SearchAck()->userId()))
   {
-    win->wprintf("%C%s%A,%Z %s %s %A(%Z%s%A) -%Z %lu %A(%Z%s%A)\n",
+    win->wprintf("%C%s%A,%Z %s %s %A(%Z%s%A) -%Z %s %A(%Z%s%A)\n",
                  COLOR_WHITE,
                  e->SearchAck()->Alias(),
                  A_BOLD, A_BOLD,
@@ -973,7 +973,7 @@ void CLicqConsole::ProcessDoneSearch(ICQEvent *e)
                  A_BOLD, A_BOLD,
                  e->SearchAck()->Email(),
                  A_BOLD, A_BOLD,
-                 e->SearchAck()->Uin(),
+        LicqUser::getUserAccountId(e->SearchAck()->userId()).c_str(),
                  A_BOLD, A_BOLD,
                  e->SearchAck()->Status() == SA_ONLINE ? "online" :
                  e->SearchAck()->Status() == SA_OFFLINE ? "offline" :
@@ -1466,15 +1466,12 @@ bool CLicqConsole::ParseMacro(char *szMacro)
 /*---------------------------------------------------------------------------
  * CLicqConsole::SaveLastUser
  *-------------------------------------------------------------------------*/
-void CLicqConsole::SaveLastUser(const string& id, unsigned long nPPID)
+void CLicqConsole::SaveLastUser(const UserId& userId)
 {
   // Save this as the last user
-  if (winMain->sLastContact.szId.empty() ||
-      !(id == winMain->sLastContact.szId &&
-      nPPID == winMain->sLastContact.nPPID))
+  if (!USERID_ISVALID(winMain->sLastContact) || userId != winMain->sLastContact)
   {
-    winMain->sLastContact.nPPID = nPPID;
-    winMain->sLastContact.szId = id;
+    winMain->sLastContact = userId;
     PrintStatus();
   }
 }
@@ -1511,16 +1508,16 @@ char *CLicqConsole::CurrentGroupName()
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_Info
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_Info(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_Info(const UserId& userId, char *)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   if (u == NULL)
     return;
 
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputInfo;
   winMain->state = STATE_QUERY;
-  winMain->data = new CData(szId, nPPID);
+  winMain->data = new CData(userId);
 
   winMain->wprintf("%C%A"
                    "(G)eneral Info\n"
@@ -1530,7 +1527,7 @@ void CLicqConsole::UserCommand_Info(const char *szId, unsigned long nPPID, char
                    "(U)pdate Info\n"
                    "for %s (%s)? %C%Z",
                    m_cColorQuery->nColor, m_cColorQuery->nAttr,
-                   u->GetAlias(), szId, COLOR_WHITE, A_BOLD);
+      u->GetAlias(), u->accountId().c_str(), COLOR_WHITE, A_BOLD);
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -1552,23 +1549,25 @@ void CLicqConsole::InputInfo(int cIn)
     switch(tolower(cIn))
     {
     case 'g':
-      PrintInfo_General(data->szId, data->nPPID);
+          PrintInfo_General(data->userId);
       break;
     case 'm':
-      PrintInfo_More(data->szId, data->nPPID);
+          PrintInfo_More(data->userId);
       break;
     case 'w':
-      PrintInfo_Work(data->szId, data->nPPID);
+          PrintInfo_Work(data->userId);
       break;
     case 'a':
-      PrintInfo_About(data->szId, data->nPPID);
+          PrintInfo_About(data->userId);
       break;
     case 'u':
+        {
       winMain->wprintf("%C%AUpdate info...", m_cColorInfo->nColor,
                        m_cColorInfo->nAttr);
-      winMain->event = licqDaemon->icqRequestMetaInfo(data->szId);
+          winMain->event = licqDaemon->requestUserInfo(data->userId);
       winMain->state = STATE_PENDING;
       return;
+        }
     case '\r':
       break;
     default:
@@ -1598,9 +1597,9 @@ void CLicqConsole::InputInfo(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_View
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_View(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_View(const UserId& userId, char *)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_W);
+  LicqUser* u = gUserManager.fetchUser(userId, LOCK_W);
   if (u == NULL)
     return;
 
@@ -1630,13 +1629,13 @@ void CLicqConsole::UserCommand_View(const char *szId, unsigned long nPPID, char
 
     // Do we want to accept the file transfer?
     if (e->SubCommand() == ICQ_CMDxSUB_FILE)
-      FileChatOffer(e, szId, nPPID);
+      FileChatOffer(e, userId);
 
     delete e;
     gUserManager.DropUser(u);
     //PrintUsers();
     //PrintStatus();
-    ProcessSignal(new CICQSignal(SIGNAL_UPDATExUSER, USER_EVENTS, szId, nPPID));
+    ProcessSignal(new LicqSignal(SIGNAL_UPDATExUSER, USER_EVENTS, userId));
   }
   else
   {
@@ -1650,20 +1649,20 @@ void CLicqConsole::UserCommand_View(const char *szId, unsigned long nPPID, char
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_Remove
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_Remove(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_Remove(const UserId& userId, char *)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   if (u == NULL)
     return;
 
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputRemove;
   winMain->state = STATE_QUERY;
-  winMain->data = new CData(szId, nPPID);
+  winMain->data = new CData(userId);
 
   winMain->wprintf("%C%ARemove %s (%s) from contact list (y/N)? %C%Z",
                    m_cColorQuery->nColor, m_cColorQuery->nAttr,
-                   u->GetAlias(), szId, COLOR_WHITE, A_BOLD);
+      u->GetAlias(), u->accountId().c_str(), COLOR_WHITE, A_BOLD);
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -1685,7 +1684,7 @@ void CLicqConsole::InputRemove(int cIn)
     // The input is done
     if (strncasecmp(data->szQuery, "yes", strlen(data->szQuery)) == 0)
     {
-      licqDaemon->RemoveUserFromList(data->szId, data->nPPID);
+        gUserManager.removeUser(data->userId);
       winMain->wprintf("%C%AUser removed.\n", m_cColorInfo->nColor,
                        m_cColorInfo->nAttr);
     }
@@ -1714,16 +1713,18 @@ void CLicqConsole::InputRemove(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_FetchAutoResponse
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_FetchAutoResponse(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_FetchAutoResponse(const UserId& userId, char *)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   winMain->wprintf("%C%AFetching auto-response for %s (%s)...",
                    m_cColorInfo->nColor, m_cColorInfo->nAttr,
-                   u->GetAlias(), szId);
+      u->GetAlias(), u->accountId().c_str());
   winMain->RefreshWin();
+  string szId = u->accountId();
+  unsigned long nPPID = u->ppid();
   gUserManager.DropUser(u);
 
-  winMain->event = licqDaemon->icqFetchAutoResponse(szId, nPPID);
+  winMain->event = licqDaemon->icqFetchAutoResponse(szId.c_str(), nPPID);
   // InputMessage just to catch the cancel key
   winMain->fProcessInput = &CLicqConsole::InputMessage;
   winMain->data = NULL;
@@ -1782,9 +1783,9 @@ int StrToRange(char *sz, int nLast, int nStart)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_History
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, char *szArg)
+void CLicqConsole::UserCommand_History(const UserId& userId, char *szArg)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   if (u == NULL)
     return;
   HistoryList lHistory;
@@ -1795,7 +1796,7 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
     return;
   }
   char *szFrom;
-  if (gUserManager.FindOwner(szId, nPPID))
+  if (gUserManager.isOwner(userId))
     szFrom = strdup("Server");
   else
     szFrom = strdup(u->GetAlias());
@@ -1876,19 +1877,19 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_Msg
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_Msg(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_Msg(const UserId& userId, char *)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   if (u == NULL)
     return;
 
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputMessage;
   winMain->state = STATE_MLE;
-  winMain->data = new DataMsg(szId, nPPID);
+  winMain->data = new DataMsg(userId);
 
   winMain->wprintf("%BEnter message to %b%s%B (%b%s%B):\n", u->GetAlias(),
-                   szId);
+      u->accountId().c_str());
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -1897,10 +1898,10 @@ void CLicqConsole::UserCommand_Msg(const char *szId, unsigned long nPPID, char *
 /*---------------------------------------------------------------------------
  * SendDirect
  *-------------------------------------------------------------------------*/
-static bool SendDirect(const char *szId, unsigned long nPPID, char c)
+static bool SendDirect(const UserId& userId, char c)
 {
   bool bDirect = (c != 's');
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   if (u != NULL)
   {
     if (u->SocketDesc(ICQ_CHNxNONE) == -1 &&
@@ -1957,12 +1958,12 @@ void CLicqConsole::InputMessage(int cIn)
       sz--;
       *sz = '\0';
       sz += 2;
-      bool bDirect = SendDirect(data->szId, data->nPPID, *sz);
+      bool bDirect = SendDirect(data->userId, *sz);
       winMain->wprintf("%C%ASending message %s...", m_cColorInfo->nColor,
                        m_cColorInfo->nAttr,
                        !bDirect ? "through the server" : "direct");
-      winMain->event = licqDaemon->ProtoSendMessage(data->szId, data->nPPID, data->szMsg,
-                       bDirect, *sz == 'u');
+      winMain->event = licqDaemon->sendMessage(data->userId, data->szMsg,
+          !bDirect, *sz == 'u');
       winMain->state = STATE_PENDING;
       break;
     }
@@ -1976,8 +1977,8 @@ void CLicqConsole::InputMessage(int cIn)
     {
       winMain->wprintf("%C%ASending message through the server...",
                        m_cColorInfo->nColor, m_cColorInfo->nAttr);
-      winMain->event = licqDaemon->ProtoSendMessage(data->szId, data->nPPID, data->szMsg,
-                       false, false);
+        winMain->event = licqDaemon->sendMessage(data->userId, data->szMsg,
+            true, false);
       winMain->state = STATE_PENDING;
     }
     else
@@ -1998,16 +1999,16 @@ void CLicqConsole::InputMessage(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_SendFile
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_SendFile(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_SendFile(const UserId& userId, char *)
 {
   // Get the file name
   winMain->fProcessInput = &CLicqConsole::InputSendFile;
   winMain->state = STATE_LE;
-  winMain->data = new DataSendFile(szId, nPPID);
+  winMain->data = new DataSendFile(userId);
 
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   winMain->wprintf("%BEnter file to send to %b%s%B (%b%s%B):\n",
-                   u->GetAlias(), szId);
+      u->GetAlias(), u->accountId().c_str());
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -2086,7 +2087,7 @@ void CLicqConsole::InputSendFile(int cIn)
 
       *sz = '\0';
       sz++;
-      bool bDirect = SendDirect(data->szId, data->nPPID, *sz);
+      bool bDirect = SendDirect(data->userId, *sz);
       bDirect = true; // XXX hack
       winMain->wprintf("%C%ASending File %s...",
                        m_cColorInfo->nColor, m_cColorInfo->nAttr,
@@ -2095,7 +2096,7 @@ void CLicqConsole::InputSendFile(int cIn)
       ConstFileList lFileList;
       lFileList.push_back(strdup(data->szFileName));
 
-      winMain->event = licqDaemon->icqFileTransfer(data->szId,
+      winMain->event = licqDaemon->fileTransferPropose(data->userId,
               data->szFileName, data->szDescription, lFileList, ICQ_TCPxMSG_NORMAL,
                        !bDirect);
       break;
@@ -2110,7 +2111,7 @@ void CLicqConsole::InputSendFile(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_SetAutoResponse
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_SetAutoResponse(const char* /* szId */, unsigned long /* nPPID */, char *)
+void CLicqConsole::UserCommand_SetAutoResponse(const UserId& /* userId */, char *)
 {
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputAutoResponse;
@@ -2168,16 +2169,16 @@ void CLicqConsole::InputAutoResponse(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_Url
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_Url(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_Url(const UserId& userId, char *)
 {
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputUrl;
   winMain->state = STATE_LE;
-  winMain->data = new DataUrl(szId, nPPID);
+  winMain->data = new DataUrl(userId);
 
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   winMain->wprintf("%BEnter URL to %b%s%B (%b%ld%B): ",
-                   u->GetAlias(), szId);
+      u->GetAlias(), u->accountId().c_str());
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -2232,13 +2233,12 @@ void CLicqConsole::InputUrl(int cIn)
       }
       *sz = '\0';
       sz++;
-      bool bDirect = SendDirect(data->szId, data->nPPID, *sz);
+      bool bDirect = SendDirect(data->userId, *sz);
       winMain->wprintf("%C%ASending URL %s...",
                        m_cColorInfo->nColor, m_cColorInfo->nAttr,
                        !bDirect ? "through the server" : "direct");
-      winMain->event = licqDaemon->ProtoSendUrl(data->szId, data->nPPID, data->szUrl,
-                                              data->szDesc,
-                                              bDirect, *sz == 'u');
+      winMain->event = licqDaemon->sendUrl(data->userId, data->szUrl,
+          data->szDesc, !bDirect, *sz == 'u');
       winMain->state = STATE_PENDING;
       break;
     }
@@ -2252,8 +2252,8 @@ void CLicqConsole::InputUrl(int cIn)
     {
       winMain->wprintf("%C%ASending URL through the server...",
                        m_cColorInfo->nColor, m_cColorInfo->nAttr);
-      winMain->event = licqDaemon->ProtoSendUrl(data->szId, data->nPPID, data->szUrl,
-                                              data->szDesc, false, false);
+        winMain->event = licqDaemon->sendUrl(data->userId, data->szUrl,
+            data->szDesc, true, false);
       winMain->state = STATE_PENDING;
     }
     else
@@ -2274,18 +2274,18 @@ void CLicqConsole::InputUrl(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_Sms
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_Sms(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_Sms(const UserId& userId, char *)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
   if (u == NULL)
     return;
 
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputSms;
   winMain->state = STATE_MLE;
-  winMain->data = new DataSms(szId, nPPID);
+  winMain->data = new DataSms(userId);
   winMain->wprintf("%BEnter SMS to %b%s%B (%b%s%B):\n", u->GetAlias(),
-                   u->GetCellularNumber());
+      u->getCellularNumber().c_str());
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -2328,11 +2328,11 @@ void CLicqConsole::InputSms(int cIn)
       }
       *sz = '\0';
       sz++;
-      ICQUser *u = gUserManager.FetchUser(data->szId, data->nPPID, LOCK_R);
+      const LicqUser* u = gUserManager.fetchUser(data->userId);
       winMain->wprintf("%C%ASending SMS to %s ...", m_cColorInfo->nColor,
-                       m_cColorInfo->nAttr, u->GetCellularNumber());
-        winMain->event = licqDaemon->icqSendSms(data->szId, LICQ_PPID,
-            u->GetCellularNumber(), data->szMsg);
+          m_cColorInfo->nAttr, u->getCellularNumber().c_str());
+        winMain->event = licqDaemon->icqSendSms(u->accountId().c_str(), u->ppid(),
+          u->getCellularNumber().c_str(), data->szMsg);
       gUserManager.DropUser(u);
       winMain->state = STATE_PENDING;
       break;
@@ -2380,14 +2380,14 @@ void CLicqConsole::InputAuthorize(int cIn)
       if (data->bUrgent)
       {
         winMain->wprintf("%C%AGranting authorizing to %s...", m_cColorInfo->nColor,
-                         m_cColorInfo->nAttr, data->szId);
-        winMain->event = licqDaemon->ProtoAuthorizeGrant(data->szId, data->nPPID, data->szMsg);
+            m_cColorInfo->nAttr, USERID_TOSTR(data->userId));
+        winMain->event = licqDaemon->authorizeGrant(data->userId, data->szMsg);
       }
       else
       {
         winMain->wprintf("%C%ARefusing authorizing to %s...", m_cColorInfo->nColor,
-                         m_cColorInfo->nAttr, data->szId);
-        winMain->event = licqDaemon->ProtoAuthorizeRefuse(data->szId, data->nPPID, data->szMsg);
+            m_cColorInfo->nAttr, USERID_TOSTR(data->userId));
+        winMain->event = licqDaemon->authorizeRefuse(data->userId, data->szMsg);
       }
 
       winMain->fProcessInput = &CLicqConsole::InputCommand;
@@ -3080,13 +3080,13 @@ void CLicqConsole::InputRegistrationWizard(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::FileChatOffer
  *-------------------------------------------------------------------------*/
-void CLicqConsole::FileChatOffer(CUserEvent *e, const char *_szId, unsigned long _nPPID)
+void CLicqConsole::FileChatOffer(CUserEvent *e, const UserId& userId)
 {
   CEventFile *f = (CEventFile *)e;
   // Get y or n
   winMain->fProcessInput = &CLicqConsole::InputFileChatOffer;
   winMain->state = STATE_QUERY;
-  winMain->data = new DataFileChatOffer(f, _szId, _nPPID);
+  winMain->data = new DataFileChatOffer(f, userId);
   winMain->wprintf("%C%ADo you wish to accept this request? (y/N) %C%Z",
                    m_cColorQuery->nColor, m_cColorQuery->nAttr, COLOR_WHITE, A_BOLD);
   winMain->RefreshWin();
@@ -3103,6 +3103,10 @@ void CLicqConsole::InputFileChatOffer(int cIn)
   char *sz;
   CEventFile *f = data->f;
 
+  const LicqUser* user = gUserManager.fetchUser(data->userId);
+  string szId = user->accountId();
+  gUserManager.DropUser(user);
+
   switch(winMain->state)
   {
   case STATE_QUERY:
@@ -3115,7 +3119,7 @@ void CLicqConsole::InputFileChatOffer(int cIn)
 
           // Make the ftman
           CFileTransferManager *ftman = new CFileTransferManager(licqDaemon,
-                                        strtoul(data->szId, (char **)NULL, 10));
+              szId.c_str());
           ftman->SetUpdatesEnabled(1);
           m_lFileStat.push_back(ftman);
 
@@ -3126,9 +3130,10 @@ void CLicqConsole::InputFileChatOffer(int cIn)
           // Accept the file
           const char *home = getenv("HOME");
           ftman->ReceiveFiles(home);
-          licqDaemon->icqFileTransferAccept(data->szId, ftman->LocalPort(),
-                                            f->Sequence(), f->MessageID(), f->IsDirect(),
-                                            f->FileDescription(), f->Filename(), f->FileSize());
+          licqDaemon->fileTransferAccept(data->userId,
+              ftman->LocalPort(), f->Sequence(), f->MessageID()[0],
+              f->MessageID()[1], f->FileDescription(), f->Filename(),
+              f->FileSize(), !f->IsDirect());
           winMain->fProcessInput = &CLicqConsole::InputCommand;
 
           if(winMain->data)
@@ -3158,13 +3163,12 @@ void CLicqConsole::InputFileChatOffer(int cIn)
       data->szReason[data->nPos - 1] = '\0';
 
       // XXX hack
-      unsigned long dummy[2] = { 0, 0 };
-      licqDaemon->icqFileTransferRefuse(data->szId, data->szReason,
-                                        f->Sequence(), dummy, true);
+      licqDaemon->fileTransferRefuse(data->userId,
+          data->szReason, f->Sequence(), 0, 0, false);
 
       // We are done now
       winMain->wprintf("%ARefusing file from %s with reason: %Z%s\n",
-                       A_BOLD, data->szId, A_BOLD, data->szReason);
+          A_BOLD, USERID_TOSTR(data->userId), A_BOLD, data->szReason);
       winMain->fProcessInput = &CLicqConsole::InputCommand;
 
       if(winMain->data)
@@ -3181,9 +3185,9 @@ void CLicqConsole::InputFileChatOffer(int cIn)
 /*-------------------------------------------------------------------------
  * CLicqConsole::UserCommand_Secure
  *-----------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_Secure(const char *szId, unsigned long nPPID, char *szStatus)
+void CLicqConsole::UserCommand_Secure(const UserId& userId, char *szStatus)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const LicqUser* u = gUserManager.fetchUser(userId);
 
   if(!licqDaemon->CryptoEnabled())
   {
@@ -3220,16 +3224,14 @@ void CLicqConsole::UserCommand_Secure(const char *szId, unsigned long nPPID, cha
     winMain->wprintf("%ARequest secure channel with %s ... ", A_BOLD,
                      u->GetAlias());
     gUserManager.DropUser(u);
-    if (nPPID == LICQ_PPID )
-      winMain->event = licqDaemon->icqOpenSecureChannel(szId);
+    winMain->event = licqDaemon->secureChannelOpen(userId);
   }
   else if(strcasecmp(szStatus, "close") == 0)
   {
     winMain->wprintf("%AClose secure channel with %s ... ", A_BOLD,
                      u->GetAlias());
     gUserManager.DropUser(u);
-    if (nPPID == LICQ_PPID )
-      winMain->event = licqDaemon->icqCloseSecureChannel(szId);
+    winMain->event = licqDaemon->secureChannelClose(userId);
   }
   else
   {
@@ -3250,7 +3252,7 @@ void CLicqConsole::UserSelect()
   winMain->state = STATE_LE;
 
   //TODO which owner
-  winMain->data = new DataUserSelect(gUserManager.OwnerId(LICQ_PPID).c_str(), LICQ_PPID);
+  winMain->data = new DataUserSelect(gUserManager.ownerUserId(LICQ_PPID));
 
   ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   winMain->wprintf("%A%CEnter your password for %s (%s):%C%Z\n", A_BOLD,
