@@ -17,18 +17,23 @@
 // written by Jon Keating <jon@licq.org>
 
 #include "msnpacket.h"
+
+#include <cctype>
+#include <cstdio>
+#include <cstring>
+#include <string>
+#include <openssl/md5.h>
+
 #include "msn_constants.h"
 
-#include <string>
-#include <stdio.h>
-#include <openssl/md5.h>
+using namespace std;
 
 static inline bool is_base64(unsigned char c)
 {
   return (isalnum(c) || (c == '+') || (c == '/'));
 }
 
-string MSN_Base64Encode(unsigned char const* szIn, unsigned int nLen)
+string MSN_Base64Encode(const char* szIn, unsigned int nLen)
 {
   string ret;
   int i = 0;
@@ -38,7 +43,7 @@ string MSN_Base64Encode(unsigned char const* szIn, unsigned int nLen)
 
   while (nLen--)
   {
-    char_array_3[i++] = *(szIn++);
+    char_array_3[i++] = static_cast<unsigned char>(*(szIn++));
     if (i == 3)
     {
       char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
@@ -620,14 +625,34 @@ CPS_MSNTypingNotification::CPS_MSNTypingNotification(const char *szEmail)
   m_pBuffer->Pack(szParams2, strlen(szParams2));
 }
 
-CPS_MSNInvitation::CPS_MSNInvitation(char *szToEmail, char *szFromEmail,
-				     char *szMSNObject)
+CPS_MSNCancelInvite::CPS_MSNCancelInvite(const string& cookie, const string& code)
+  : CMSNPayloadPacket(false)
+{
+  m_szCommand = strdup("MSG");
+
+  char payload[512];
+  snprintf(payload, sizeof(payload),
+      "MIME-Version: 1.0\r\n"
+      "Content-Type: text/x-msmsgsinvite; charset=UTF-8\r\n"
+      "\r\n"
+      "Invitation-Command: CANCEL\r\n"
+      "Cancel-Code: %s\r\n"
+      "Invitation-Cookie: %s\r\n"
+      "\r\n",
+      code.c_str(), cookie.c_str());
+
+  m_nPayloadSize = strlen(payload);
+  CMSNPayloadPacket::InitBuffer();
+  m_pBuffer->Pack(payload, m_nPayloadSize);
+}
+
+CPS_MSNInvitation::CPS_MSNInvitation(const char* szToEmail,
+    const char* szFromEmail, const char* szMSNObject)
   : CMSNP2PPacket(szToEmail)
 {
   char *szBranchGUID = CreateGUID();
   m_szCallGUID = CreateGUID();
-  string strMSNObject64 = MSN_Base64Encode((unsigned char *)szMSNObject,
-    strlen(szMSNObject));
+  string strMSNObject64 = MSN_Base64Encode(szMSNObject, strlen(szMSNObject));
 
   char szBodyBuf[512];
   char szHeaderBuf[512];
@@ -652,9 +677,9 @@ CPS_MSNInvitation::CPS_MSNInvitation(char *szToEmail, char *szFromEmail,
 	   "Call-ID: {%s}\r\n"
 	   "Max-Forwards: 0\r\n"
 	   "Content-Type: application/x-msnmsgr-sessionreqbody\r\n"
-	   "Content-Length: %d\r\n"
+	   "Content-Length: %lu\r\n"
 	   "\r\n", szToEmail, szToEmail, szFromEmail, szBranchGUID,
-	   m_szCallGUID, strlen(szBodyBuf)+1);
+	   m_szCallGUID, static_cast<unsigned long>(strlen(szBodyBuf)+1));
 
   string strMsg = szHeaderBuf;
   strMsg += szBodyBuf;
@@ -675,10 +700,9 @@ CPS_MSNInvitation::CPS_MSNInvitation(char *szToEmail, char *szFromEmail,
   m_pBuffer->PackUnsignedLong(0);
 }
 
-CPS_MSNP2PBye::CPS_MSNP2PBye(const char *_szToEmail, const char *_szFromEmail,
-			     const char *_szCallId, unsigned long _nBaseId,
-			     unsigned long _nAckId, unsigned long _nDataSizeHI,
-			     unsigned long _nDataSizeLO)
+CPS_MSNP2PBye::CPS_MSNP2PBye(const char *_szToEmail, const char* _szFromEmail,
+    const char* _szCallId, unsigned long _nBaseId, unsigned long _nAckId,
+    unsigned long /* _nDataSizeHI */, unsigned long /* _nDataSizeLO */)
   : CMSNP2PPacket(_szToEmail, 0, _nBaseId, 0, 0, 0, 4, 0, 0, _nAckId, 0, 0, 0)
 		     //SizeHI, _nDataSizeLO)
 {
