@@ -19,18 +19,15 @@
 #ifndef __MSNPACKET_H
 #define __MSNPACKET_H
 
+#include <cstdlib>
+#include <pthread.h>
+#include <string>
+
 #include "licq_packets.h"
 
 #include "msnbuffer.h"
 
-#include <pthread.h>
-#include <stdlib.h>
-
-#include <string>
-
-using std::string;
-
-static const string base64_chars = 
+static const std::string base64_chars =
              "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
              "abcdefghijklmnopqrstuvwxyz"
              "0123456789+/";
@@ -293,10 +290,17 @@ public:
   CPS_MSNTypingNotification(const char *);
 };
 
+class CPS_MSNCancelInvite : public CMSNPayloadPacket
+{
+public:
+  CPS_MSNCancelInvite(const std::string& cookie, const std::string& code);
+};
+
 class CPS_MSNInvitation : public CMSNP2PPacket
 {
 public:
-  CPS_MSNInvitation(char *szToEmail, char *szFromEmail, char *szMSNObject);
+  CPS_MSNInvitation(const char* szToEmail, const char* szFromEmail,
+      const char* szMSNObject);
 };
 
 class CPS_MSNP2PBye : public CMSNP2PPacket
