@@ -1,6 +1,6 @@
 /*
  * This file is part of Licq, an instant messaging client for UNIX.
- * Copyright (C) 1999-2006 Licq developers
+ * Copyright (C) 1999-2009 Licq developers
  *
  * Licq is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ signals:
   void signal_updateGroups();
 
 private slots:
-  void listUpdated(CICQSignal* sig);
+  void listUpdated(unsigned long subSignal);
 
 private:
   unsigned short currentGroupId() const;
