@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 2 -*-
 /*
  * This file is part of Licq, an instant messaging client for UNIX.
- * Copyright (C) 1999-2006 Licq developers
+ * Copyright (C) 1999-2009 Licq developers
  *
  * Licq is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,16 +49,15 @@
 
 
 //-----Constructor------------------------------------------------------------
-CFileDlg::CFileDlg(const char *szId, unsigned long nPPID, CICQDaemon *daemon,
+CFileDlg::CFileDlg(const UserId& userId, CICQDaemon *daemon,
   QWidget* parent)
   : QWidget(parent, "FileDialog", WDestructiveClose)
 {
   // If we are the server, then we are receiving a file
-  m_szId = szId ? strdup(szId) : 0;
-  m_nPPID = nPPID;
+  myUserId = userId;
   licqDaemon = daemon;
 
-  setCaption(tr("Licq - File Transfer (%1)").arg(m_szId));
+  setCaption(tr("Licq - File Transfer (%1)").arg(USERID_TOSTR(myUserId)));
 
   unsigned short CR = 0;
   QGridLayout* lay = new QGridLayout(this, 8, 3, 8, 8);
@@ -125,7 +124,7 @@ CFileDlg::CFileDlg(const char *szId, unsigned long nPPID, CICQDaemon *daemon,
   connect(btnCancel, SIGNAL(clicked()), this, SLOT(close()));
 
   //TODO fix this
-  ftman = new CFileTransferManager(licqDaemon, strtoul(m_szId, (char **)NULL, 10));
+  ftman = new CFileTransferManager(licqDaemon, LicqUser::getUserAccountId(myUserId).c_str());
   ftman->SetUpdatesEnabled(2);
   sn = new QSocketNotifier(ftman->Pipe(), QSocketNotifier::Read);
   connect(sn, SIGNAL(activated(int)), SLOT(slot_ft()));
@@ -263,7 +262,7 @@ void CFileDlg::slot_ft()
   char buf[32];
   read(ftman->Pipe(), buf, 32);
 
-  QTextCodec *codec = UserCodec::codecForProtoUser(m_szId, m_nPPID);
+  const QTextCodec* codec = UserCodec::codecForUserId(myUserId);
 
   CFileTransferEvent *e = NULL;
   while ( (e = ftman->PopFileTransferEvent()) != NULL)
@@ -444,8 +443,8 @@ bool CFileDlg::GetLocalFileName()
   QString f;
   bool bValid = false;
 
-  QTextCodec *codec = UserCodec::codecForProtoUser(m_szId, m_nPPID);
-  
+  const QTextCodec* codec = UserCodec::codecForUserId(myUserId);
+
   // Get the local filename and open it, loop until valid or cancel
   while(!bValid)
   {
