@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 2 -*-
 /*
  * This file is part of Licq, an instant messaging client for UNIX.
- * Copyright (C) 1999-2006 Licq developers
+ * Copyright (C) 1999-2009 Licq developers
  *
  * Licq is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 #include <QDialog>
 
+#include <licq_types.h>
+
 class QCheckBox;
 class QLineEdit;
 
@@ -36,7 +38,7 @@ class AddUserDlg : public QDialog
    Q_OBJECT
 
 public:
-   AddUserDlg(QString id = QString::null, unsigned long ppid = 0, QWidget* parent = 0);
+   AddUserDlg(const UserId& userId = USERID_NONE, QWidget* parent = 0);
 
 private:
    QCheckBox* myNotify;
