@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 2 -*-
 /*
  * This file is part of Licq, an instant messaging client for UNIX.
- * Copyright (C) 2007 Licq developers
+ * Copyright (C) 2007-2009 Licq developers
  *
  * Licq is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,10 @@
 #include <QSpinBox>
 #include <QVBoxLayout>
 
+#ifndef USE_KDE
+# include <QStyleFactory>
+#endif
+
 #include <licq_icqd.h>
 
 #include "config/chat.h"
@@ -82,26 +86,30 @@ QWidget* Settings::ContactList::createPageContactList(QWidget* parent)
   myShowDividersCheck->setToolTip(tr("Show the \"--online--\" and \"--offline--\" bars in the contact list"));
   myAppearanceLayout->addWidget(myShowDividersCheck, 2, 0);
 
+  myMode2ViewCheck = new QCheckBox(tr("Alternate threaded view"));
+  myMode2ViewCheck->setToolTip(tr("Separate online and offline users in threaded view"));
+  myAppearanceLayout->addWidget(myMode2ViewCheck, 3, 0);
+
   myFontStylesCheck = new QCheckBox(tr("Use font styles"));
   myFontStylesCheck->setToolTip(tr("Use italics and bold in the user list to "
       "indicate special characteristics such as online notify and visible list"));
-  myAppearanceLayout->addWidget(myFontStylesCheck, 3, 0);
+  myAppearanceLayout->addWidget(myFontStylesCheck, 4, 0);
 
   myShowExtIconsCheck = new QCheckBox(tr("Show extended icons"));
   myShowExtIconsCheck->setToolTip(tr("Show birthday, invisible, and custom auto response icons to the right of users in the list"));
-  myAppearanceLayout->addWidget(myShowExtIconsCheck, 4, 0);
+  myAppearanceLayout->addWidget(myShowExtIconsCheck, 5, 0);
 
   myShowPhoneIconsCheck = new QCheckBox(tr("Show phone icons"));
   myShowPhoneIconsCheck->setToolTip(tr("Show extended icons for phone statuses"));
   connect(myShowExtIconsCheck, SIGNAL(toggled(bool)),
       myShowPhoneIconsCheck, SLOT(setEnabled(bool)));
-  myAppearanceLayout->addWidget(myShowPhoneIconsCheck, 5, 0);
+  myAppearanceLayout->addWidget(myShowPhoneIconsCheck, 6, 0);
 
   myShowUserIconsCheck = new QCheckBox(tr("Show user display picture"));
   myShowUserIconsCheck->setToolTip(tr("Show the user's display picture"
       " instead of a status icon, if the user"
       " is online and has a display picture"));
-  myAppearanceLayout->addWidget(myShowUserIconsCheck, 6, 0);
+  myAppearanceLayout->addWidget(myShowUserIconsCheck, 7, 0);
 
   myAlwaysShowONUCheck = new QCheckBox(tr("Always show online notify users"));
   myAlwaysShowONUCheck->setToolTip(tr("Show online notify users who are offline even when offline users are hidden."));
@@ -135,6 +143,20 @@ QWidget* Settings::ContactList::createPageContactList(QWidget* parent)
   myFrameStyleLayout->addWidget(myFrameStyleEdit);
   myAppearanceLayout->addLayout(myFrameStyleLayout, 5, 1);
 
+#ifndef USE_KDE
+  QHBoxLayout* guiStyleLayout = new QHBoxLayout();
+
+  myGuiStyleLabel = new QLabel(tr("GUI Style:"));
+  myGuiStyleLabel->setToolTip(tr("Select look and feel for the GUI. Available styles may vary between systems."));
+  guiStyleLayout->addWidget(myGuiStyleLabel);
+  myGuiStyleCombo = new QComboBox();
+  myGuiStyleCombo->setToolTip(myGuiStyleLabel->toolTip());
+  foreach (QString style, QStyleFactory::keys())
+    myGuiStyleCombo->addItem(style);
+  guiStyleLayout->addWidget(myGuiStyleCombo);
+  myAppearanceLayout->addLayout(guiStyleLayout, 6, 1);
+#endif
+
   // Make the columns evenly wide, otherwise the QLineEdit steals the space
   myAppearanceLayout->setColumnStretch(0, 1);
   myAppearanceLayout->setColumnStretch(1, 1);
@@ -340,6 +362,7 @@ void Settings::ContactList::load()
   myMainWinStickyCheck->setChecked(generalConfig->mainwinSticky());
 
   myGridLinesCheck->setChecked(contactListConfig->showGridLines());
+  myMode2ViewCheck->setChecked(contactListConfig->mode2View());
   myFontStylesCheck->setChecked(contactListConfig->useFontStyles());
   myShowExtIconsCheck->setChecked(contactListConfig->showExtendedIcons());
   myShowPhoneIconsCheck->setChecked(contactListConfig->showPhoneIcons());
@@ -393,6 +416,13 @@ void Settings::ContactList::load()
   myTransparentCheck->setChecked(Config::Skin::active()->frame.transparent);
   myFrameStyleEdit->setText(QString::number(static_cast<int>(Config::Skin::active()->frame.frameStyle)));
 
+#ifndef USE_KDE
+  QString currentStyle = generalConfig->guiStyle();
+  for (int i = 0; i < myGuiStyleCombo->count(); ++i)
+    if (myGuiStyleCombo->itemText(i).compare(currentStyle, Qt::CaseInsensitive) == 0)
+      myGuiStyleCombo->setCurrentIndex(i);
+#endif
+
   myAutoUpdateInfoCheck->setChecked(gLicqDaemon->AutoUpdateInfo());
   myAutoUpdateInfoPluginsCheck->setChecked(gLicqDaemon->AutoUpdateInfoPlugins());
   myAutoUpdateStatusPluginsCheck->setChecked(gLicqDaemon->AutoUpdateStatusPlugins());
@@ -414,6 +444,7 @@ void Settings::ContactList::apply()
   generalConfig->setMainwinSticky(myMainWinStickyCheck->isChecked());
 
   contactListConfig->setShowGridLines(myGridLinesCheck->isChecked());
+  contactListConfig->setMode2View(myMode2ViewCheck->isChecked());
   contactListConfig->setUseFontStyles(myFontStylesCheck->isChecked());
   contactListConfig->setShowExtendedIcons(myShowExtIconsCheck->isChecked());
   contactListConfig->setShowPhoneIcons(myShowPhoneIconsCheck->isChecked());
@@ -457,6 +488,10 @@ void Settings::ContactList::apply()
   Config::Skin::active()->setFrameTransparent(myTransparentCheck->isChecked());
   Config::Skin::active()->setFrameStyle(myFrameStyleEdit->text().toUShort());
 
+#ifndef USE_KDE
+  generalConfig->setGuiStyle(myGuiStyleCombo->currentText());
+#endif
+
   gLicqDaemon->SetAutoUpdateInfo(myAutoUpdateInfoCheck->isChecked());
   gLicqDaemon->SetAutoUpdateInfoPlugins(myAutoUpdateInfoPluginsCheck->isChecked());
   gLicqDaemon->SetAutoUpdateStatusPlugins(myAutoUpdateStatusPluginsCheck->isChecked());
